@@ -1,5 +1,25 @@
+"""LLMling: main package.
+
+LLM simplified.
+"""
+
 from __future__ import annotations
 
+from importlib.metadata import version
+
+__version__ = version("llmling")
+__title__ = "LLMling"
+
+__author__ = "Philipp Temminghoff"
+__author_email__ = "philipptemminghoff@googlemail.com"
+__copyright__ = "Copyright (c) 2024 Philipp Temminghoff"
+__license__ = "MIT"
+__url__ = "https://github.com/phil65/llmling"
+
+from upathtools import register_http_filesystems
+
+register_http_filesystems()
+
 from llmling.resources import (
     ResourceLoader,
     LoadedResource,
@@ -32,7 +52,6 @@ from llmling.config.models import (
 from llmling.config.store import ConfigStore
 from llmling.core.baseregistry import BaseRegistry
 
-__version__ = "1.6.10"
 
 __all__ = [
     "BasePrompt",
@@ -58,5 +77,6 @@ __all__ = [
     "RuntimeConfig",
     "StaticPrompt",
     "ToolError",
+    "__version__",
     "resource_registry",
 ]
