@@ -8,20 +8,17 @@ from llmling.cli.constants import (
     output_format_opt,
     verbose_opt,
 )
-from llmling.cli.utils import format_output, get_command_help
+from llmling.cli.utils import OutputFormat, format_output, get_command_help
 
 
-PROMPTS_HELP = "Prompt management commands."
-prompts_cli = t.Typer(
-    help=get_command_help(PROMPTS_HELP),
-    no_args_is_help=True,
-)
+help_cmd = get_command_help("Prompt management commands.")
+prompts_cli = t.Typer(help=help_cmd, no_args_is_help=True)
 
 
 @prompts_cli.command("list")
 def list_prompts(
     config_path: str = config_file_opt,
-    output_format: str = output_format_opt,
+    output_format: OutputFormat = output_format_opt,
     verbose: bool = verbose_opt,
 ):
     """List available prompts."""
@@ -35,7 +32,7 @@ def list_prompts(
 def show_prompt(
     config_path: str = config_file_opt,
     name: str = t.Argument(help=PROMPT_NAME_HELP),
-    output_format: str = output_format_opt,
+    output_format: OutputFormat = output_format_opt,
     verbose: bool = verbose_opt,
 ):
     """Show prompt details."""
