@@ -2,33 +2,37 @@
 
 from __future__ import annotations
 
-from collections.abc import Sequence as TypingSequence  # noqa: TC003
+from collections.abc import Sequence
 import inspect
-import os  # noqa: TC003
-from typing import Annotated, Any, Literal, Self
+import os
+from typing import TYPE_CHECKING, Annotated, Any, Literal, Self
 import warnings
 
 from pydantic import (
-    BaseModel,
     ConfigDict,
     Field,
+    HttpUrl,  # noqa: TC002
     SecretStr,
     field_validator,
     model_validator,
 )
+from schemez import Schema
 
 from llmling import config_resources
 from llmling.config.base import ConfigModel
 from llmling.core import exceptions
 from llmling.core.log import get_logger
-from llmling.core.typedefs import ProcessingStep  # noqa: TC001
+from llmling.core.typedefs import ProcessingStep
 from llmling.processors.base import ProcessorConfig  # noqa: TC001
 from llmling.prompts.models import PromptType  # noqa: TC001
-from llmling.tools.toolsets import ToolSet
 from llmling.utils.importing import import_callable, import_class
 from llmling.utils.paths import guess_mime_type
 
 
+if TYPE_CHECKING:
+    from upath.types import JoinablePathLike
+
+
 ResourceType = Literal["path", "text", "cli", "source", "callable"]
 LogLevel = Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
 
@@ -155,16 +159,16 @@ class Jinja2Config(ConfigModel):
 class GlobalSettings(ConfigModel):
     """Global settings that apply to all components."""
 
-    timeout: int = 30
+    timeout: int = Field(default=30, ge=0)
     """Maximum time in seconds to wait for operations"""
 
-    max_retries: int = 3
+    max_retries: int = Field(default=3, ge=0)
     """Maximum number of retries for failed operations"""
 
     requirements: list[str] = Field(default_factory=list)
     """List of package requirments for the functions used in this file."""
 
-    pip_index_url: str | None = None
+    pip_index_url: HttpUrl | None = None
     """Alternative PyPI index URL for package installation"""
 
     extra_paths: list[str] = Field(default_factory=list)
@@ -186,7 +190,7 @@ class GlobalSettings(ConfigModel):
     """Control which system capabilities are exposed to LLMs."""
 
 
-class BaseResource(BaseModel):
+class BaseResource(Schema):
     """Base class for all resource types."""
 
     type: str = Field(init=False)
@@ -208,7 +212,7 @@ class BaseResource(BaseModel):
     """Technical identifier (automatically set from config key during registration)."""
 
     # TODO: proper extra="forbid" for all subclasses.
-    model_config = ConfigDict(frozen=True, use_attribute_docstrings=True)
+    model_config = ConfigDict(frozen=True)
 
     def validate_resource(self) -> list[str]:
         """Validate resource at runtime.
@@ -266,23 +270,22 @@ class PathResource(BaseResource):
 
     def validate_resource(self) -> list[str]:
         """Check if path exists for local files."""
-        import upath
+        from upathtools import to_upath
 
-        warnings = []
-        path = upath.UPath(self.path)
+        path = to_upath(self.path)
         prefixes = ("http://", "https://")
-
-        if not path.exists() and not path.as_uri().startswith(prefixes):
-            warnings.append(f"Resource path not found: {path}")
-
-        return warnings
+        return [
+            f"Resource path not found: {path}"
+            for path in [path]
+            if not path.exists() and not path.as_uri().startswith(prefixes)
+        ]
 
     @property
     def supports_watching(self) -> bool:
         """Whether this resource instance supports watching."""
-        import upath
+        from upathtools import to_upath
 
-        path = upath.UPath(self.path)
+        path = to_upath(self.path)
         if not path.exists():
             msg = f"Cannot watch non-existent path: {self.path}"
             warnings.warn(msg, UserWarning, stacklevel=2)
@@ -345,7 +348,7 @@ class CLIResource(BaseResource):
     type: Literal["cli"] = Field(default="cli", init=False)
     """Discriminator field identifying this as a CLI-based resource."""
 
-    command: str | TypingSequence[str]
+    command: str | Sequence[str]
     """Command to execute (string or sequence of arguments)."""
 
     shell: bool = False
@@ -354,7 +357,7 @@ class CLIResource(BaseResource):
     cwd: str | None = None
     """Working directory for command execution."""
 
-    timeout: float | None = None
+    timeout: float | None = Field(None, ge=0)
     """Maximum time in seconds to wait for command completion."""
 
     @model_validator(mode="after")
@@ -396,10 +399,11 @@ class RepositoryResource(BaseResource):
     """Optional password for authentication."""
 
     def validate_resource(self) -> list[str]:
-        warnings = []
-        if self.user and not self.password:
-            warnings.append(f"Repository {self.repo_url} has user but no password")
-        return warnings
+        return [
+            f"Repository {self.repo_url} has user but no password"
+            for _ in [None]
+            if self.user and not self.password
+        ]
 
 
 class SourceResource(BaseResource):
@@ -472,6 +476,23 @@ class WatchConfig(ConfigModel):
     """Path to .gitignore-style file"""
 
 
+class ToolHints(ConfigModel):
+    """Configuration for tool execution hints."""
+
+    read_only: bool | None = None
+    """Hints that this tool only reads data without modifying anything"""
+
+    destructive: bool | None = None
+    """Hints that this tool performs destructive operations that cannot be undone"""
+
+    idempotent: bool | None = None
+    """Hints that this tool has idempotent behaviour"""
+
+    open_world: bool | None = None
+    """Hints that this tool can access / interact with external resources beyond the
+    current system"""
+
+
 class ToolConfig(ConfigModel):
     """Configuration for a tool."""
 
@@ -484,13 +505,13 @@ class ToolConfig(ConfigModel):
     description: str | None = None
     """Optional override for the tool's description"""
 
+    hints: ToolHints = ToolHints()
+    """Hints about the tool's behavior and execution characteristics"""
+
 
 class BaseToolsetConfig(ConfigModel):
     """Base configuration for toolsets."""
 
-    namespace: str | None = Field(default=None)
-    """Optional namespace prefix for tool names"""
-
 
 class OpenAPIToolsetConfig(BaseToolsetConfig):
     """Configuration for OpenAPI toolsets."""
@@ -527,6 +548,8 @@ class CustomToolsetConfig(BaseToolsetConfig):
     @field_validator("import_path", mode="after")
     @classmethod
     def validate_import_path(cls, v: str) -> str:
+        from llmling.tools.toolsets import ToolSet
+
         # v is already confirmed to be a str here
         try:
             cls = import_class(v)
@@ -577,7 +600,6 @@ class Config(ConfigModel):
         frozen=True,
         arbitrary_types_allowed=True,
         extra="allow",  # extra fields used by the server for example.
-        use_attribute_docstrings=True,
     )
 
     # @model_validator(mode="before")
@@ -624,7 +646,7 @@ class Config(ConfigModel):
                     raise ValueError(msg)
 
     @classmethod
-    def from_file(cls, path: str | os.PathLike[str]) -> Self:
+    def from_file(cls, path: JoinablePathLike) -> Self:
         """Load configuration from YAML file.
 
         This function only handles the basic loading and model validation.
