@@ -7,17 +7,18 @@ This module provides the RuntimeConfig class which represents the fully initiali
 from __future__ import annotations
 
 from contextlib import asynccontextmanager, contextmanager
+import importlib
+import os
 from typing import TYPE_CHECKING, Any, Literal, Self
 
-import depkit
-import logfire
+import upath
 from upath import UPath
 
 from llmling.config.manager import ConfigManager
-from llmling.config.models import BaseResource, PathResource
+from llmling.config.models import Config, PathResource
 from llmling.config.utils import prepare_runtime, toolset_config_to_toolset
 from llmling.core import exceptions
-from llmling.core.events import EventEmitter
+from llmling.core.chain import ChainTool
 from llmling.core.log import get_logger
 from llmling.core.typedefs import ProcessingStep
 from llmling.processors.jinjaprocessor import Jinja2Processor
@@ -31,17 +32,15 @@ from llmling.resources.registry import ResourceRegistry
 from llmling.tools.base import LLMCallableTool
 from llmling.tools.exceptions import ToolError
 from llmling.tools.registry import ToolRegistry
-from llmling.utils import importing
 
 
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator, Callable, Iterator, Sequence
-    import os
     import types
 
-    from llmling.completions.types import CompletionFunction
-    from llmling.config.models import Config
-    from llmling.core.events import RegistryEvents
+    from upath.types import JoinablePathLike
+
+    from llmling.config.models import BaseResource
     from llmling.processors.base import ProcessorResult
     from llmling.prompts.models import BasePrompt
     from llmling.resources.models import LoadedResource
@@ -51,7 +50,7 @@ logger = get_logger(__name__)
 RegistryType = Literal["resource", "prompt", "tool"]
 
 
-class RuntimeConfig(EventEmitter):
+class RuntimeConfig:
     """Fully initialized runtime configuration.
 
     This represents the "live" state of a Config, with all components
@@ -79,6 +78,8 @@ class RuntimeConfig(EventEmitter):
             prompt_registry: Registry for prompts
             tool_registry: Registry for tools
         """
+        import depkit
+
         super().__init__()
         self._config = config
         self._loader_registry = loader_registry
@@ -89,13 +90,13 @@ class RuntimeConfig(EventEmitter):
         self._initialized = False
         # Register builtin processors
         proc = Jinja2Processor(config.global_settings.jinja_environment)
-        self._processor_registry.register("jinja_template", proc)
+        self._processor_registry["jinja_template"] = proc
         settings = self._config.global_settings
         self._dep_manager = depkit.DependencyManager(
             prefer_uv=settings.prefer_uv,
             requirements=settings.requirements,
             extra_paths=settings.extra_paths,
-            pip_index_url=settings.pip_index_url,
+            pip_index_url=str(settings.pip_index_url) if settings.pip_index_url else None,
             scripts=settings.scripts,
         )
 
@@ -125,10 +126,14 @@ class RuntimeConfig(EventEmitter):
 
     async def __aenter__(self) -> Self:
         """Initialize dependencies and registries."""
-        await self._dep_manager.__aenter__()
-        if not self._initialized:
+        # Check and set initialized flag atomically before ANY async op
+        # This should allow caeses where same runtime gets initialized multiple times
+        needs_init = not self._initialized
+        self._initialized = True
+
+        if needs_init:
+            await self._dep_manager.__aenter__()
             await self._initialize_registries()
-            self._initialized = True
         return self
 
     async def __aexit__(
@@ -157,43 +162,31 @@ class RuntimeConfig(EventEmitter):
                 tool_config.import_path,
                 name_override=tool_config.name,
                 description_override=tool_config.description,
+                hints=tool_config.hints,
             )
 
         self._initialize_toolsets()
 
-        for name, prompt_config in self._config.prompts.items():
-            if isinstance(prompt_config, DynamicPrompt):
+        for name, prompt in self._config.prompts.items():
+            if isinstance(prompt, DynamicPrompt):
                 # Convert completion function import paths to actual functions
-                if completions := prompt_config.completions:
-                    completion_funcs: dict[str, CompletionFunction] = {}
-                    for arg_name, import_path in completions.items():
-                        try:
-                            func = importing.import_callable(import_path)
-                            completion_funcs[arg_name] = func
-                        except ValueError:
-                            msg = "Failed to import completion function for %s: %s"
-                            logger.warning(msg, arg_name, import_path)
-                    prompt_config.completions = completion_funcs  # type: ignore
-                args, desc = extract_function_info(
-                    prompt_config.import_path, prompt_config.completions
-                )
-                prompt_config.arguments = args
-                if not prompt_config.description:
-                    prompt_config.description = desc
+                # TODO: we are setting .completions to a non-allowed type here
+                prompt.completions = prompt.get_completion_functions()  # type: ignore
+                args, desc = extract_function_info(prompt.import_path, prompt.completions)
+                prompt.arguments = args
+                if not prompt.description:
+                    prompt.description = desc
 
-            self._prompt_registry[name] = prompt_config
+            self._prompt_registry[name] = prompt
 
     def _initialize_toolsets(self) -> None:
         """Initialize toolsets from config."""
         for name, config in self._config.toolsets.items():
             try:
                 toolset = toolset_config_to_toolset(config)
-                # Get tool prefix
-                prefix = f"{config.namespace}." if config.namespace else f"{name}."
-
-                # Register all tools
                 for tool in toolset.get_llm_callable_tools():
-                    tool_name = f"{prefix}{tool.name}"
+                    # tool_name = f"{name}.{tool.name}"
+                    tool_name = tool.name
                     if tool_name in self._tool_registry:
                         msg = "Tool %s from toolset %s overlaps with existing tool"
                         logger.warning(msg, tool.name, name)
@@ -229,7 +222,7 @@ class RuntimeConfig(EventEmitter):
     @asynccontextmanager
     async def open(
         cls,
-        source: str | os.PathLike[str] | Config,
+        source: JoinablePathLike | Config,
         *,
         validate: bool = True,
         strict: bool = False,
@@ -258,14 +251,8 @@ class RuntimeConfig(EventEmitter):
 
         Example:
             ```python
-            # Using a config file:
             async with RuntimeConfig.open("config.yml") as runtime:
                 resource = await runtime.load_resource("example")
-
-            # Using an existing Config object:
-            config = Config(...)
-            async with RuntimeConfig.open(config) as runtime:
-                resource = await runtime.load_resource("example")
             ```
 
         Note:
@@ -280,7 +267,7 @@ class RuntimeConfig(EventEmitter):
     @contextmanager
     def open_sync(
         cls,
-        source: str | os.PathLike[str] | Config,
+        source: JoinablePathLike | Config,
         *,
         validate: bool = True,
         strict: bool = False,
@@ -309,14 +296,8 @@ class RuntimeConfig(EventEmitter):
 
         Example:
             ```python
-            # Using a config file:
             with RuntimeConfig.open_sync("config.yml") as runtime:
                 resource = runtime.load_resource_sync("example")
-
-            # Using an existing Config object:
-            config = Config(...)
-            with RuntimeConfig.open_sync(config) as runtime:
-                resource = runtime.load_resource_sync("example")
             ```
 
         Note:
@@ -328,7 +309,7 @@ class RuntimeConfig(EventEmitter):
             yield runtime
 
     @classmethod
-    def from_file(cls, path: str | os.PathLike[str]) -> Self:
+    def from_file(cls, path: JoinablePathLike) -> Self:
         """Convenience function to directly create runtime config from a file.
 
         Args:
@@ -341,8 +322,7 @@ class RuntimeConfig(EventEmitter):
         return cls.from_config(manager.config)
 
     @classmethod
-    @logfire.instrument("Creating runtime configuration")
-    def from_config(cls, config: Config) -> Self:
+    def from_config(cls, config: Config | JoinablePathLike) -> Self:
         """Create a fully initialized runtime config from static config.
 
         Args:
@@ -351,6 +331,8 @@ class RuntimeConfig(EventEmitter):
         Returns:
             Initialized runtime configuration
         """
+        if isinstance(config, str | os.PathLike | UPath):
+            config = ConfigManager.load(config).config
         loader_registry = ResourceLoaderRegistry()
         processor_registry = ProcessorRegistry()
         resource_registry = ResourceRegistry(
@@ -359,7 +341,7 @@ class RuntimeConfig(EventEmitter):
         )
         prompt_registry = PromptRegistry()
         tool_registry = ToolRegistry()
-
+        assert isinstance(config, Config)
         runtime = cls(
             config=config,
             loader_registry=loader_registry,
@@ -368,12 +350,24 @@ class RuntimeConfig(EventEmitter):
             prompt_registry=prompt_registry,
             tool_registry=tool_registry,
         )
-        if config.global_settings.enable_resource_tools:
-            runtime._register_resource_tools()
+        from llmling.config.llm_tools import LLMTools
 
-        if config.global_settings.enable_tool_management:
-            runtime._register_tool_management_tools()
+        llm_tools = LLMTools(runtime)
+        capabilities = config.global_settings.llm_capabilities
 
+        # Register enabled capabilities
+        if capabilities.load_resource:
+            tool_registry["load_resource"] = llm_tools.load_resource
+        if capabilities.get_resources:
+            tool_registry["get_resources"] = llm_tools.get_resources
+        if capabilities.install_package:
+            tool_registry["install_package"] = llm_tools.install_package
+        if capabilities.register_tool:
+            tool_registry["register_tool"] = llm_tools.register_tool
+        if capabilities.register_code_tool:
+            tool_registry["register_code_tool"] = llm_tools.register_code_tool
+        if capabilities.chain_tools:
+            tool_registry["chain"] = ChainTool(runtime)
         return runtime
 
     async def startup(self) -> None:
@@ -390,50 +384,18 @@ class RuntimeConfig(EventEmitter):
         await self._tool_registry.shutdown()
         await self._processor_registry.shutdown()
 
-    def _register_resource_tools(self) -> None:
-        """Register resource tools if enabled."""
-        # Resource management
-        tool = LLMCallableTool.from_callable(self.get_resources)
-        self._tool_registry.register("get_resources", tool)
-
-        # Resource loading
-        tool = LLMCallableTool.from_callable(self.load_resource)
-        self._tool_registry.register("load_resource", tool)
-
-    def _register_tool_management_tools(self) -> None:
-        """Register tools for dynamic tool management and dependencies."""
-        # Tool registration tools
-        tool = LLMCallableTool.from_callable(self.register_tool)
-        self._tool_registry.register("register_tool", tool)
-
-        # Code-based tool registration
-        tool = LLMCallableTool.from_callable(self.register_code_tool)
-        self._tool_registry.register("register_code_tool", tool)
-
-        # Package management
-        tool = LLMCallableTool.from_callable(self.install_package)
-        self._tool_registry.register("install_package", tool)
-
-        # Tool discovery
-        tool = LLMCallableTool.from_callable(self.get_tools)
-        self._tool_registry.register("list_tools", tool)
-
     # Resource Management
     async def load_resource(self, name: str) -> LoadedResource:
-        """Load the content of a resource by its name.
-
-        Use this tool to load the actual content of a resource. First use
-        get_resources() to see what's available, then provide the resource name
-        from that list.
+        """Load a resource by its registered name.
 
         Args:
-            name: Name of the resource to load (must match a name from get_resources())
+            name: Name of the resource as registered in configuration
 
         Returns:
-            The loaded resource including its content and metadata.
+            LoadedResource containing content and metadata
 
         Raises:
-            ResourceError: If the resource doesn't exist or can't be loaded.
+            ResourceError: If resource doesn't exist or cannot be loaded
         """
         return await self._resource_registry.load(name)
 
@@ -462,7 +424,7 @@ class RuntimeConfig(EventEmitter):
             if name in self._resource_registry:
                 return uri_or_name, self._resource_registry[name]
             # Create temporary resource for the URI
-            resource: BaseResource = PathResource(path=uri_or_name)
+            resource: BaseResource = PathResource(path=uri_or_name)  # pyright: ignore
             return uri_or_name, resource
 
         # 2. Try as resource name
@@ -481,7 +443,7 @@ class RuntimeConfig(EventEmitter):
         if "/" in uri_or_name or "\\" in uri_or_name or "." in uri_or_name:
             try:
                 logger.debug("Trying as file path")
-                resource = PathResource(path=uri_or_name)
+                resource = PathResource(path=uri_or_name)  # pyright: ignore
                 loader = PathResourceLoader.create(resource, uri_or_name)
                 uri = loader.create_uri(name=uri_or_name)
             except Exception as exc:  # noqa: BLE001
@@ -538,11 +500,12 @@ class RuntimeConfig(EventEmitter):
         """
         return self._resource_registry.get_uri(name)
 
-    def get_resource(self, name: str) -> BaseResource:
-        """Get a resource configuration by name.
+    def get_resource(self, name_or_uri: str) -> BaseResource:
+        """Get a resource configuration by name or URI.
 
         Args:
-            name: Name of the resource to get
+            name_or_uri: Name of the resource or its URI. URIs must start with a scheme
+                (e.g., "file://", "text://", etc.)
 
         Returns:
             The resource configuration
@@ -550,22 +513,74 @@ class RuntimeConfig(EventEmitter):
         Raises:
             ResourceError: If resource not found
         """
-        return self._resource_registry[name]
+        if "://" in name_or_uri:  # It's a URI
+            for resource in self._resource_registry.values():
+                if resource.uri == name_or_uri:
+                    return resource
+            msg = f"No resource found with URI: {name_or_uri}"
+            raise exceptions.ResourceError(msg)
+        return self._resource_registry[name_or_uri]
 
     def get_resources(self) -> Sequence[BaseResource]:
-        """List all available resources and their metadata.
-
-        This tool returns information about all resources that can be loaded, including:
-        - name: The identifier to use with load_resource
-        - description: What the resource contains
-        - uri: The resource's location
-        - mimeType: The type of content (e.g., text/markdown, application/json)
+        """Get all registered resource configurations.
 
         Returns:
-            List of resources with their complete metadata.
+            List of all resource configurations
         """
         return list(self._resource_registry.values())
 
+    async def register_resource_by_loader(
+        self,
+        loader_type: str,
+        name: str,
+        *,
+        description: str = "",
+        **params: Any,
+    ) -> BaseResource:
+        """Register a resource using a loader type.
+
+        Provides a parameter-based interface for registering resources
+        without requiring resource objects.
+
+        Args:
+            loader_type: Type of loader to use ("path", "text", "cli", etc.)
+            name: Name for the resource
+            description: Optional description
+            **params: Loader-specific parameters
+
+        Example:
+            await runtime.register_resource_by_loader(
+                "path",
+                "readme",
+                description="Project readme",
+                path="README.md"
+            )
+
+            await runtime.register_resource_by_loader(
+                "text",
+                "greeting",
+                content="Hello, world!"
+            )
+        """
+        try:
+            # Get loader class from registry
+            loader_cls = self._loader_registry[loader_type]
+
+            # Get the resource class from the loader
+            resource_class = loader_cls.context_class
+
+            # Create resource instance with just the essential fields
+            resource = resource_class(description=description, **params)
+            self._resource_registry.register(name, resource)
+        except KeyError as exc:
+            msg = f"Unknown loader type: {loader_type}"
+            raise exceptions.ResourceError(msg) from exc
+        except Exception as exc:
+            msg = f"Failed to create resource: {exc}"
+            raise exceptions.ResourceError(msg) from exc
+        else:
+            return resource
+
     def register_resource(
         self,
         name: str,
@@ -617,11 +632,15 @@ class RuntimeConfig(EventEmitter):
         """
         return dict(self._tool_registry)
 
-    async def execute_tool(self, name: str, **params: Any) -> Any:
+    async def execute_tool(
+        self,
+        _name: str,  # prefixed with "_" to avoid name collisions
+        **params: Any,
+    ) -> Any:
         """Execute a tool by name.
 
         Args:
-            name: Name of the tool to execute
+            _name: Name of the tool to execute
             **params: Parameters to pass to the tool
 
         Returns:
@@ -630,7 +649,7 @@ class RuntimeConfig(EventEmitter):
         Raises:
             ToolError: If tool execution fails
         """
-        return await self._tool_registry.execute(name, **params)
+        return await self._tool_registry.execute(_name, **params)
 
     def get_tool(self, name: str) -> LLMCallableTool:
         """Get a tool by name.
@@ -660,11 +679,10 @@ class RuntimeConfig(EventEmitter):
     ) -> str:
         """Install a Python package using the dependency manager.
 
-        This allows installing packages needed for tool functionality.
         Package specifications follow PIP format (e.g. "requests>=2.28.0").
 
         Args:
-            package: Package specification to install (e.g. "requests", "pillow>=10.0.0")
+            package: Package specification to install
 
         Returns:
             Message confirming the installation
@@ -672,12 +690,11 @@ class RuntimeConfig(EventEmitter):
         Raises:
             ToolError: If installation fails or package spec is invalid
 
-        Examples:
-            >>> await install_package("requests")
-            >>> await install_package("pillow>=10.0.0")
+        Example:
+            >>> await runtime.install_package("requests>=2.28.0")
         """
         try:
-            await self._dep_manager.install_dependency(package)
+            self._dep_manager.install_dependency(package)
             return f"Successfully installed package: {package}"  # noqa: TRY300
         except Exception as exc:
             msg = f"Failed to install package: {exc}"
@@ -685,42 +702,50 @@ class RuntimeConfig(EventEmitter):
 
     async def register_tool(
         self,
-        name: str,
         function: str | Callable[..., Any],
+        name: str | None = None,
         description: str | None = None,
+        *,
+        auto_install: bool = False,
     ) -> str:
         """Register a new tool from a function or import path.
 
-        This tool can register a callable either from:
-        - A direct callable object (function/method)
-        - An import path as string (e.g. "webbrowser.open")
-
         Args:
-            name: Name for the new tool
             function: Function to register (callable or import path)
-            description: Optional description override (uses function docstring if None)
+            name: Optional name override (uses function name if None)
+            description: Optional description override (uses docstring if None)
+            auto_install: Whether to attempt installing missing package
 
         Returns:
-            Message confirming the tool registration
+            Message confirming the registration
 
         Raises:
-            ToolError: If registration fails (e.g. invalid import path, invalid function)
+            ToolError: If registration fails
 
-        Examples:
-            >>> await register_tool("open_url", "webbrowser.open")
-            >>> await register_tool("my_tool", my_function)
+        Example:
+            >>> await runtime.register_tool("webbrowser.open", "open_url")
+            >>> await runtime.register_tool(my_function)
         """
         try:
+            if auto_install and isinstance(function, str):
+                # Try to import root module, install if missing
+                package = function.split(".")[0]
+                try:
+                    importlib.import_module(package)
+                except ImportError:
+                    logger.info("Attempting to install package: %s", package)
+                    await self.install_package(package)
+
             tool = LLMCallableTool.from_callable(
                 function, name_override=name, description_override=description
             )
-            self._tool_registry.register(name, tool)
+            self._tool_registry.register(name or tool.name, tool)
             source = function if isinstance(function, str) else function.__name__
         except Exception as exc:
             msg = f"Failed to register tool: {exc}"
             raise ToolError(msg) from exc
         else:
-            return f"Successfully registered tool: {name} from {source}"
+            return f"Successfully registered tool: {tool.name} from {source}"
 
     async def register_code_tool(
         self,
@@ -737,23 +762,21 @@ class RuntimeConfig(EventEmitter):
         Args:
             name: Name for the new tool
             code: Python code defining the tool function
-            description: Optional description override (uses function docstring if None)
+            description: Optional description override
 
         Returns:
-            Message confirming the tool registration
+            Message confirming the registration
 
         Raises:
-            ToolError: If registration fails (e.g. invalid code, no callable found)
+            ToolError: If registration fails
 
-        Examples:
-            >>> await register_code_tool(
-            ...     name="word_count",
-            ...     code='''
-            ...     async def count_words(text: str) -> dict[str, int]:
-            ...         words = text.split()
-            ...         return {"total": len(words)}
-            ...     '''
-            ... )
+        Example:
+            >>> code = '''
+            ... async def count_words(text: str) -> dict[str, int]:
+            ...     words = text.split()
+            ...     return {"total": len(words)}
+            ... '''
+            >>> await runtime.register_code_tool("word_count", code)
         """
         try:
             namespace: dict[str, Any] = {}
@@ -871,11 +894,7 @@ class RuntimeConfig(EventEmitter):
                     first_line[:100] + "..." if len(first_line) > 100 else first_line  # noqa: PLR2004
                 )
 
-            prompt = StaticPrompt(
-                name=name,
-                description=description,
-                messages=messages,
-            )
+            prompt = StaticPrompt(name=name, description=description, messages=messages)
             self._prompt_registry.register(name, prompt, replace=replace)
 
         except Exception as exc:
@@ -885,7 +904,7 @@ class RuntimeConfig(EventEmitter):
     def register_file_prompt(
         self,
         name: str,
-        path: str | os.PathLike[str],
+        path: JoinablePathLike,
         *,
         description: str,
         output_format: Literal["text", "markdown", "jinja2"] = "text",
@@ -910,14 +929,16 @@ class RuntimeConfig(EventEmitter):
             ...     format="markdown"
             ... )
         """
+        from upathtools import to_upath
+
         try:
-            if not UPath(path).exists():
+            if not to_upath(path).exists():
                 msg = f"Prompt file not found: {path}"
                 raise FileNotFoundError(msg)  # noqa: TRY301
 
             prompt = FilePrompt(
                 name=name,
-                path=path,
+                path=upath.UPath(path),
                 description=description,
                 format=output_format,
                 watch=watch,
@@ -1046,46 +1067,6 @@ class RuntimeConfig(EventEmitter):
             content, [ProcessingStep(name=processor_name, kwargs=kwargs)]
         )
 
-    # Registry Observation
-
-    def add_observer(
-        self,
-        observer: RegistryEvents[str, Any],
-        registry_type: RegistryType,
-    ) -> None:
-        """Add an observer for registry changes.
-
-        Args:
-            observer: Observer to add
-            registry_type: Type of registry to observe
-        """
-        match registry_type:
-            case "resource":
-                self._resource_registry.add_observer(observer)
-            case "prompt":
-                self._prompt_registry.add_observer(observer)
-            case "tool":
-                self._tool_registry.add_observer(observer)
-
-    def remove_observer(
-        self,
-        observer: RegistryEvents[str, Any],
-        registry_type: RegistryType,
-    ) -> None:
-        """Remove a registry observer.
-
-        Args:
-            observer: Observer to remove
-            registry_type: Type of registry to remove from
-        """
-        match registry_type:
-            case "resource":
-                self._resource_registry.remove_observer(observer)
-            case "prompt":
-                self._prompt_registry.remove_observer(observer)
-            case "tool":
-                self._tool_registry.remove_observer(observer)
-
     @property
     def original_config(self) -> Config:
         """Get the original static configuration.
