@@ -3,14 +3,13 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-import json
 from typing import TYPE_CHECKING, TypedDict
 
 from llmling.core.log import get_logger
 
 
 if TYPE_CHECKING:
-    import os
+    from upath.types import JoinablePathLike
 
 
 logger = get_logger(__name__)
@@ -58,11 +57,13 @@ class ConfigStore:
 
     def load_mapping(self) -> ConfigMapping:
         """Load config mapping from disk."""
+        import anyenv
+
         if not self.config_file.exists():
             return ConfigMapping(configs={}, active=None)
         try:
             text = self.config_file.read_text("utf-8")
-            data = json.loads(text)
+            data = anyenv.load_json(text, return_type=dict)
             active = data.get("active")
             configs = data.get("configs", {})
             return ConfigMapping(configs=configs, active=active)
@@ -72,12 +73,14 @@ class ConfigStore:
 
     def save_mapping(self, mapping: ConfigMapping) -> None:
         """Save config mapping to disk."""
+        import anyenv
+
         try:
-            self.config_file.write_text(json.dumps(mapping, indent=2))
+            self.config_file.write_text(anyenv.dump_json(mapping, indent=True))
         except Exception:
             logger.exception("Failed to save config mapping")
 
-    def add_config(self, name: str, path: str | os.PathLike[str]) -> None:
+    def add_config(self, name: str, path: JoinablePathLike) -> None:
         """Add a new named config.
 
         Args:
@@ -91,13 +94,13 @@ class ConfigStore:
             IsADirectoryError: If path points to a directory
         """
         # Basic validation
-        from upath import UPath
+        from upathtools import to_upath
 
         if not name.isidentifier():
             msg = f"Invalid config name: {name} (must be a valid Python identifier)"
             raise ValueError(msg)
 
-        path_obj = UPath(path).resolve()
+        path_obj = to_upath(path).resolve()
         if not path_obj.exists():
             msg = f"Config file not found: {path}"
             raise FileNotFoundError(msg)
