@@ -3,9 +3,10 @@
 from __future__ import annotations
 
 import os
-from typing import TYPE_CHECKING, TypeVar
+from typing import TYPE_CHECKING
 
 from pydantic import BaseModel
+from upath import UPath
 
 from llmling.config.manager import ConfigManager
 from llmling.config.models import (
@@ -22,21 +23,22 @@ from llmling.utils import importing
 
 
 if TYPE_CHECKING:
+    from upath.types import JoinablePathLike
+
     from llmling.config.runtime import RuntimeConfig
     from llmling.tools.toolsets import ToolSet
 
 logger = get_logger(__name__)
-T = TypeVar("T", bound=BaseModel)
 
 
 def toolset_config_to_toolset(config) -> ToolSet:
     match config:
-        case OpenAPIToolsetConfig():
-            return OpenAPITools(spec=config.spec, base_url=config.base_url or "")
-        case EntryPointToolsetConfig():
-            return EntryPointTools(config.module)
-        case CustomToolsetConfig():
-            toolset_class = importing.import_class(config.import_path)
+        case OpenAPIToolsetConfig(spec=spec, base_url=base_url):
+            return OpenAPITools(spec=spec, base_url=base_url or "")
+        case EntryPointToolsetConfig(module=module):
+            return EntryPointTools(module)
+        case CustomToolsetConfig(import_path=import_path):
+            toolset_class = importing.import_class(import_path)
             return toolset_class()
         case _:
             msg = f"Unknown toolset type: {type(config)}"
@@ -45,7 +47,7 @@ def toolset_config_to_toolset(config) -> ToolSet:
 
 def prepare_runtime(
     runtime_cls: type[RuntimeConfig],
-    source: str | os.PathLike[str] | Config,
+    source: JoinablePathLike | Config,
     *,
     validate: bool = True,
     strict: bool = False,
@@ -66,7 +68,7 @@ def prepare_runtime(
         ConfigError: If validation fails in strict mode
     """
     match source:
-        case str() | os.PathLike():
+        case str() | os.PathLike() | UPath():
             manager = ConfigManager.load(source, validate=validate, strict=strict)
             config = manager.config
         case Config():
@@ -85,7 +87,7 @@ def prepare_runtime(
     return runtime_cls.from_config(config)
 
 
-def merge_models(base: T, overlay: T) -> T:
+def merge_models[T: BaseModel](base: T, overlay: T) -> T:
     """Deep merge two Pydantic models."""
     if not isinstance(overlay, type(base)):
         msg = f"Cannot merge different types: {type(base)} and {type(overlay)}"
