@@ -1,3 +1,7 @@
+"""Prompt classes and registry for defining and managing prompts."""
+
+from __future__ import annotations
+
 from llmling.prompts.models import (
     BasePrompt,
     DynamicPrompt,
