@@ -5,8 +5,6 @@ from __future__ import annotations
 import os
 from typing import TYPE_CHECKING, Any
 
-import upath
-
 from llmling.config.models import PathResource, TextResource
 from llmling.core import exceptions
 from llmling.core.baseregistry import BaseRegistry
@@ -26,16 +24,16 @@ logger = get_logger(__name__)
 class ResourceLoaderRegistry(BaseRegistry[str, ResourceLoader[Any]]):
     """Registry for context loaders."""
 
-    # def __init__(self) -> None:
-    #     """Initialize with default loaders."""
-    #     super().__init__()
-    #     # Always register defaults
-    #     self.register("image", ImageResourceLoader)
-    #     self.register("path", PathResourceLoader)
-    #     self.register("text", TextResourceLoader)
-    #     self.register("cli", CLIResourceLoader)
-    #     self.register("source", SourceResourceLoader)
-    #     self.register("callable", CallableResourceLoader)
+    def __init__(self, register_default_loaders: bool = False, **kwargs) -> None:
+        """Initialize registry.
+
+        Args:
+            register_default_loaders: Register default loaders on initialization.
+            kwargs: Additional keyword arguments passed to the parent class.
+        """
+        super().__init__(**kwargs)
+        if register_default_loaders:
+            self.register_default_loaders()
 
     @property
     def _error_class(self) -> type[exceptions.LoaderError]:
@@ -49,7 +47,6 @@ class ResourceLoaderRegistry(BaseRegistry[str, ResourceLoader[Any]]):
         from llmling.resources import (
             CallableResourceLoader,
             CLIResourceLoader,
-            ImageResourceLoader,
             PathResourceLoader,
             RepositoryResourceLoader,
             SourceResourceLoader,
@@ -61,7 +58,6 @@ class ResourceLoaderRegistry(BaseRegistry[str, ResourceLoader[Any]]):
         self["cli"] = CLIResourceLoader
         self["source"] = SourceResourceLoader
         self["callable"] = CallableResourceLoader
-        self["image"] = ImageResourceLoader
         self["repository"] = RepositoryResourceLoader
 
     def get_uri_templates(self) -> list[dict[str, Any]]:
@@ -93,13 +89,15 @@ class ResourceLoaderRegistry(BaseRegistry[str, ResourceLoader[Any]]):
 
     def _validate_item(self, item: Any) -> ResourceLoader[Any]:
         """Validate and possibly transform item before registration."""
+        from upathtools import to_upath
+
         match item:
             case str() if "\n" in item:
                 resource = TextResource(content=item)
                 return TextResourceLoader.create(resource=resource, name="inline-text")
-            case str() | os.PathLike() if upath.UPath(item).exists():
+            case str() | os.PathLike() if to_upath(item).exists():
                 path_resource = PathResource(path=str(item))
-                name = upath.UPath(item).name
+                name = to_upath(item).name
                 return PathResourceLoader.create(resource=path_resource, name=name)
             case type() if issubclass(item, ResourceLoader):
                 return item()
@@ -111,4 +109,4 @@ class ResourceLoaderRegistry(BaseRegistry[str, ResourceLoader[Any]]):
 
     def get_loader(self, context: BaseResource) -> ResourceLoader[Any]:
         """Get a loader instance for a context type."""
-        return self.get(context.resource_type)
+        return self.get(context.type)
