@@ -9,17 +9,18 @@ from typing import TYPE_CHECKING, ClassVar
 from llmling.config.models import RepositoryResource
 from llmling.core import exceptions
 from llmling.core.log import get_logger
-from llmling.resources.base import LoaderContext, ResourceLoader, create_loaded_resource
+from llmling.resources.base import ResourceLoader, create_loaded_resource
 from llmling.utils.paths import guess_mime_type
 
 
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator
-    import os
 
     from git.repo import Repo
+    from upath.types import JoinablePathLike
 
     from llmling.processors.registry import ProcessorRegistry
+    from llmling.resources.base import LoaderContext
     from llmling.resources.models import LoadedResource
 
 
@@ -55,14 +56,14 @@ class RepositoryResourceLoader(ResourceLoader[RepositoryResource]):
 
     def _create_resource(
         self,
-        path: str | os.PathLike[str],
+        path: JoinablePathLike,
         name: str,
         resource: RepositoryResource,
     ) -> LoadedResource:
         """Create LoadedResource from file."""
-        import upath
+        from upathtools import to_upath
 
-        path_obj = upath.UPath(path)
+        path_obj = to_upath(path)
         try:
             content = path_obj.read_text("utf-8")
             description = f"Repository content from {resource.repo_url} ({resource.ref})"
@@ -91,7 +92,7 @@ class RepositoryResourceLoader(ResourceLoader[RepositoryResource]):
     ) -> AsyncIterator[LoadedResource]:
         """Load git content."""
         import git
-        import upath
+        from upathtools import to_upath
 
         try:
             repo = self._get_cached_repo(resource.repo_url)
@@ -111,7 +112,7 @@ class RepositoryResourceLoader(ResourceLoader[RepositoryResource]):
             # Switch to requested ref
             repo.git.checkout(resource.ref)
 
-            base_path = upath.UPath(repo.working_dir) / resource.path.lstrip("/")
+            base_path = to_upath(repo.working_dir) / resource.path.lstrip("/")
             if base_path.is_file():
                 loaded = self._create_resource(base_path, name, resource)
                 if processor_registry and resource.processors:
