@@ -4,14 +4,13 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from upath import UPath
-
 from llmling.core.log import get_logger
 
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
-    import os
+
+    from upath.types import JoinablePathLike
 
 
 logger = get_logger(__name__)
@@ -19,7 +18,7 @@ logger = get_logger(__name__)
 
 def load_patterns(
     patterns: Sequence[str] | None = None,
-    ignore_file: str | os.PathLike[str] | None = None,
+    ignore_file: JoinablePathLike | None = None,
 ) -> list[str]:
     """Load and combine watch patterns.
 
@@ -30,6 +29,8 @@ def load_patterns(
     Returns:
         Combined list of patterns
     """
+    from upathtools import to_upath
+
     result: list[str] = []
 
     # Add configured patterns
@@ -39,8 +40,7 @@ def load_patterns(
     # Add patterns from file
     if ignore_file:
         try:
-            path = UPath(ignore_file)
-            if path.exists():
+            if (path := to_upath(ignore_file)).exists():
                 # Filter empty lines and comments
                 file_patterns = [
                     line.strip()
